import os
import json
from flask import Flask, request, Response, jsonify
from flask_limiter import Limiter
from flask_cors import CORS
import werkzeug
from webargs import fields, validate
from webargs.flaskparser import use_args
import sentry_sdk
from util import util
from api import (
    accounts,
    users,
    projects,
    objects,
    snippets,
    uploads,
    groups,
    search,
    invitations,
    root,
    activitypub,
)

app = Flask(__name__)
CORS(app)
limiter = Limiter(app, default_limits=["20 per minute"])

if os.environ.get("SENTRY_DSN"):
    sentry_sdk.init(
        dsn=os.environ["SENTRY_DSN"],
        traces_sample_rate=1.0,
        profiles_sample_rate=0.1,
    )


@app.errorhandler(werkzeug.exceptions.TooManyRequests)
def handle_429(e):
    return jsonify(
        {
            "message": "You're making too many requests. Please wait for a few minutes before trying again.",
            "Allowed limit": e.description,
        }
    ), 429


@app.errorhandler(werkzeug.exceptions.BadRequest)
def handle_bad_request(e):
    return jsonify({"message": e.description}), 400


@app.errorhandler(werkzeug.exceptions.Unauthorized)
def handle_not_authorized(e):
    return jsonify({"message": e.description}), 401


@app.errorhandler(werkzeug.exceptions.Forbidden)
def handle_forbidden(e):
    return jsonify({"message": e.description}), 403


@app.errorhandler(werkzeug.exceptions.NotFound)
def handle_not_found(e):
    return jsonify({"message": e.description}), 404


@app.errorhandler(werkzeug.exceptions.UnprocessableEntity)
def handle_unprocessable_entity(e):
    validation_errors = e.data.get("messages")
    message = ""

    def build_message(message, d):
        if not d:
            return message
        for key in d:
            if isinstance(d[key], dict):
                message += f"""{str(key)}: """
                return build_message(message, d[key])
            elif isinstance(d[key], list):
                message += f"""{str(key)}: {',\n '.join(d[key])}\n"""
        return message

    if validation_errors:
        message = build_message("", validation_errors.get("json"))
    return jsonify(
        {
            "message": message,
            "validations": validation_errors,
        }
    ), 422


@app.route("/debug-sentry")
def trigger_error():
    division_by_zero = 1 / 0  # noqa: F841


# ACCOUNTS


@limiter.limit("5 per minute", key_func=util.limit_by_client, methods=["POST"])
@app.route("/accounts/register", methods=["POST"])
@use_args(
    {
        "username": fields.Str(required=True, validate=validate.Length(min=3)),
        "email": fields.Email(required=True),
        "password": fields.Str(required=True, validate=validate.Length(min=8)),
        "howFindUs": fields.Str(required=False),
    }
)
def register(args):
    return util.jsonify(
        accounts.register(
            args["username"],
            args["email"],
            args["password"],
            args.get("howFindUs"),
        )
    )


@limiter.limit("5 per minute", key_func=util.limit_by_client, methods=["POST"])
@app.route("/accounts/login", methods=["POST"])
@use_args(
    {
        "email": fields.Str(required=True),  # Can be username
        "password": fields.Str(required=True),
    }
)
def login(args):
    return util.jsonify(accounts.login(args["email"], args["password"]))


@app.route("/accounts/logout", methods=["POST"])
def logout():
    return util.jsonify(accounts.logout(util.get_user()))


@app.route("/accounts", methods=["DELETE"])
@use_args(
    {
        "password": fields.Str(required=True),
    }
)
def delete_account(args):
    return util.jsonify(accounts.delete(util.get_user(), args.get("password")))


@app.route("/accounts/email", methods=["PUT"])
@use_args(
    {
        "email": fields.Email(required=True),
    }
)
def email_address(args):
    return util.jsonify(accounts.update_email(util.get_user(), args))


@limiter.limit("5 per minute", key_func=util.limit_by_user, methods=["POST"])
@app.route("/accounts/password", methods=["PUT"])
@use_args(
    {
        "newPassword": fields.Str(required=True, validate=validate.Length(min=8)),
        "currentPassword": fields.Str(),
        "token": fields.Str(),
    }
)
def password(args):
    return util.jsonify(accounts.update_password(util.get_user(required=False), args))


@limiter.limit("5 per minute", key_func=util.limit_by_client, methods=["POST"])
@app.route("/accounts/password/reset", methods=["POST"])
@use_args(
    {
        "email": fields.Email(required=True),
    }
)
def reset_password(args):
    return util.jsonify(accounts.reset_password(args))


@app.route("/accounts/pushToken", methods=["PUT"])
@use_args(
    {
        "pushToken": fields.Str(required=True),
    }
)
def set_push_token(args):
    return util.jsonify(accounts.update_push_token(util.get_user(), args))


# UPLOADS


@app.route("/uploads/file/request", methods=["GET"])
@use_args(
    {
        "name": fields.Str(required=True),
        "size": fields.Int(required=True),
        "type": fields.Str(required=True),
        "forType": fields.Str(required=True),
        "forId": fields.Str(required=True),
    },
    location="query",
)
def file_request(args):
    file_name = args.get("name")
    file_size = args.get("size")
    file_type = args.get("type")
    for_type = args.get("forType")
    for_id = args.get("forId")
    return util.jsonify(
        uploads.generate_file_upload_request(
            util.get_user(), file_name, file_size, file_type, for_type, for_id
        )
    )


# USERS


@app.route("/users/me", methods=["GET"])
def users_me():
    return util.jsonify(users.me(util.get_user()))


@app.route("/users/<username>", methods=["GET"])
def users_username_get(username):
    return util.jsonify(users.get(util.get_user(required=False), username))


@app.route("/users/<username>", methods=["PUT"])
@use_args(
    {
        "username": fields.Str(validate=validate.Length(min=3)),
        "avatar": fields.Str(),
        "bio": fields.Str(),
        "location": fields.Str(),
        "website": fields.Str(),
        "twitter": fields.Str(),
        "facebook": fields.Str(),
        "linkedIn": fields.Str(),
        "instagram": fields.Str(),
    }
)
def users_username_put(args, username):
    return util.jsonify(users.update(util.get_user(), username, args))


@app.route("/users/<username>/feed", methods=["GET"])
def users_feed(username):
    return util.jsonify(users.get_feed(util.get_user(), username))


@app.route("/users/<username>/followers", methods=["POST", "DELETE"])
def users_followers(username):
    if request.method == "POST":
        return util.jsonify(users.create_follower(util.get_user(), username))
    if request.method == "DELETE":
        return util.jsonify(users.delete_follower(util.get_user(), username))


@app.route("/users/<username>/tours/<tour>", methods=["PUT"])
@use_args(
    {
        "status": fields.Str(
            required=True, validate=validate.OneOf(["completed", "skipped"])
        ),
    },
    location="query",
)
def users_tour(args, username, tour):
    status = args.get("status", "completed")
    return util.jsonify(users.finish_tour(util.get_user(), username, tour, status))


@app.route("/users/me/projects", methods=["GET"])
def me_projects_route():
    user = util.get_user()
    return util.jsonify({"projects": users.get_projects(user, user["_id"])})


@app.route(
    "/users/<username>/subscriptions/email/<subscription>", methods=["PUT", "DELETE"]
)
def user_email_subscription(username, subscription):
    if request.method == "PUT":
        return util.jsonify(
            users.create_email_subscription(util.get_user(), username, subscription)
        )
    if request.method == "DELETE":
        return util.jsonify(
            users.delete_email_subscription(util.get_user(), username, subscription)
        )


# PROJECTS


@app.route("/projects", methods=["POST"])
@use_args(
    {
        "name": fields.Str(required=True),
        "description": fields.Str(),
        "visibility": fields.Str(validate=validate.OneOf(["public", "private"])),
        "openSource": fields.Bool(),
        "groupVisibility": fields.List(fields.Str()),
    }
)
def projects_route(args):
    return util.jsonify(projects.create(util.get_user(), args))


@app.route("/projects/<username>/<project_path>/objects", methods=["GET"])
def project_objects_get(username, project_path):
    return util.jsonify(
        {
            "objects": projects.get_objects(
                util.get_user(required=False), username, project_path
            )
        }
    )


@app.route("/projects/<username>/<project_path>/objects", methods=["POST"])
@use_args(
    {
        "name": fields.Str(),
        "storedName": fields.Str(),
        "wif": fields.Str(),
        "type": fields.Str(required=True, validate=validate.OneOf(["file", "pattern"])),
    }
)
def project_objects_post(args, username, project_path):
    return util.jsonify(
        projects.create_object(util.get_user(), username, project_path, args)
    )


@app.route("/projects/<username>/<project_path>", methods=["GET", "DELETE"])
def project_by_path(username, project_path):
    if request.method == "GET":
        return util.jsonify(
            projects.get(util.get_user(required=False), username, project_path)
        )
    if request.method == "DELETE":
        return util.jsonify(projects.delete(util.get_user(), username, project_path))


@app.route("/projects/<username>/<project_path>", methods=["PUT"])
@use_args(
    {
        "name": fields.Str(),
        "description": fields.Str(),
        "visibility": fields.Str(validate=validate.OneOf(["public", "private"])),
        "openSource": fields.Bool(),
        "groupVisibility": fields.List(fields.Str()),
    }
)
def project_by_path_put(args, username, project_path):
    return util.jsonify(projects.update(util.get_user(), username, project_path, args))


# OBJECTS


@app.route("/objects/<id>", methods=["GET", "DELETE"])
def objects_route(id):
    if request.method == "GET":
        return util.jsonify(objects.get(util.get_user(required=False), id))
    if request.method == "DELETE":
        return util.jsonify({"_id": objects.delete(util.get_user(), id)})


@app.route("/objects/<id>", methods=["PUT"])
@use_args(
    {
        "name": fields.Str(),
        "description": fields.Str(),
        "pattern": fields.Dict(),
    }
)
def objects_route_put(args, id):
    return util.jsonify(objects.update(util.get_user(), id, args))


@app.route("/objects/<id>/projects/<project_id>", methods=["PUT"])
def copy_object_route(id, project_id):
    return util.jsonify(objects.copy_to_project(util.get_user(), id, project_id))


@app.route("/objects/<id>/wif", methods=["GET"])
def objects_get_wif(id):
    return util.jsonify(objects.get_wif(util.get_user(required=False), id))


@app.route("/objects/<id>/pdf", methods=["GET"])
def objects_get_pdf(id):
    return util.jsonify(objects.get_pdf(util.get_user(required=False), id))


@app.route("/objects/<id>/comments", methods=["GET"])
def object_comments_get(id):
    return util.jsonify(objects.get_comments(util.get_user(required=False), id))


@app.route("/objects/<id>/comments", methods=["POST"])
@use_args(
    {
        "content": fields.Str(required=True, validate=validate.Length(min=1)),
    }
)
def object_comments_post(args, id):
    return util.jsonify(objects.create_comment(util.get_user(), id, args))


@app.route("/objects/<id>/comments/<comment_id>", methods=["DELETE"])
def object_comment(id, comment_id):
    return util.jsonify(objects.delete_comment(util.get_user(), id, comment_id))


# SNIPPETS
@app.route("/snippets", methods=["GET"])
def snippets_route_get():
    return util.jsonify(snippets.list_for_user(util.get_user()))


@app.route("/snippets", methods=["POST"])
@use_args(
    {
        "name": fields.Str(required=True, validate=validate.Length(min=3)),
        "type": fields.Str(required=True, validate=validate.OneOf(["warp", "weft"])),
        "threading": fields.List(fields.Dict(), allow_none=True),
        "treadling": fields.List(fields.Dict(), allow_none=True),
    }
)
def snippets_route_post(args):
    return util.jsonify(snippets.create(util.get_user(), args))


@app.route("/snippets/<id>", methods=["DELETE"])
def snippet_route(id):
    return util.jsonify(snippets.delete(util.get_user(), id))


# GROUPS


@app.route("/groups", methods=["GET"])
def groups_route_get():
    return util.jsonify(groups.get(util.get_user(required=True)))


@app.route("/groups", methods=["POST"])
@use_args(
    {
        "name": fields.Str(required=True, validate=validate.Length(min=3)),
        "description": fields.Str(),
        "closed": fields.Bool(),
    }
)
def groups_route_post(args):
    return util.jsonify(groups.create(util.get_user(required=True), args))


@app.route("/groups/<id>", methods=["GET", "DELETE"])
def group_route(id):
    if request.method == "GET":
        return util.jsonify(groups.get_one(util.get_user(required=False), id))
    if request.method == "DELETE":
        return util.jsonify(groups.delete(util.get_user(required=True), id))


@app.route("/groups/<id>", methods=["PUT"])
@use_args(
    {
        "name": fields.Str(),
        "description": fields.Str(),
        "closed": fields.Bool(),
    }
)
def group_route_put(args, id):
    return util.jsonify(groups.update(util.get_user(required=True), id, args))


@app.route("/groups/<id>/entries", methods=["GET"])
def group_entries_route_get(id):
    return util.jsonify(groups.get_entries(util.get_user(required=True), id))


@app.route("/groups/<id>/entries", methods=["GET", "POST"])
@use_args(
    {
        "content": fields.Str(required=True, validate=validate.Length(min=1)),
        "attachments": fields.List(fields.Dict()),
    }
)
def group_entries_route_post(args, id):
    return util.jsonify(groups.create_entry(util.get_user(required=True), id, args))


@app.route("/groups/<id>/entries/<entry_id>", methods=["DELETE"])
def group_entry_route(id, entry_id):
    return util.jsonify(groups.delete_entry(util.get_user(required=True), id, entry_id))


@app.route("/groups/<id>/entries/<entry_id>/replies", methods=["POST"])
@use_args(
    {
        "content": fields.Str(required=True, validate=validate.Length(min=1)),
        "attachments": fields.List(fields.Dict()),
    }
)
def group_entry_replies_route(args, id, entry_id):
    return util.jsonify(
        groups.create_entry_reply(util.get_user(required=True), id, entry_id, args)
    )


@app.route("/groups/<id>/entries/<entry_id>/replies/<reply_id>", methods=["DELETE"])
def group_entry_reply_route(id, entry_id, reply_id):
    return util.jsonify(
        groups.delete_entry_reply(util.get_user(required=True), id, entry_id, reply_id)
    )


@app.route("/groups/<id>/members", methods=["GET"])
def group_members_route(id):
    return util.jsonify(groups.get_members(util.get_user(required=True), id))


@app.route("/groups/<id>/members/<user_id>", methods=["PUT", "DELETE"])
def group_member_route(id, user_id):
    if request.method == "DELETE":
        return util.jsonify(
            groups.delete_member(util.get_user(required=True), id, user_id)
        )
    if request.method == "PUT":
        return util.jsonify(
            groups.create_member(util.get_user(required=True), id, user_id)
        )


@app.route("/groups/<id>/projects", methods=["GET"])
def group_projects_route(id):
    return util.jsonify(groups.get_projects(util.get_user(required=True), id))


@app.route("/groups/<id>/invitations", methods=["GET"])
def group_invites_route_get(id):
    return util.jsonify(
        invitations.get_group_invitations(util.get_user(required=True), id)
    )


@app.route("/groups/<id>/invitations", methods=["POST"])
@use_args(
    {
        "user": fields.Str(required=True),
    }
)
def group_invites_route(args, id):
    return util.jsonify(
        invitations.create_group_invitation(util.get_user(required=True), id, args)
    )


@app.route("/groups/<id>/invitations/<invite_id>", methods=["DELETE"])
def group_invite_route(id, invite_id):
    return util.jsonify(
        invitations.delete_group_invitation(util.get_user(required=True), id, invite_id)
    )


@app.route("/groups/<id>/requests", methods=["POST", "GET"])
def group_requests_route(id):
    if request.method == "POST":
        return util.jsonify(
            invitations.create_group_request(util.get_user(required=True), id)
        )
    if request.method == "GET":
        return util.jsonify(
            invitations.get_group_requests(util.get_user(required=True), id)
        )


# SEARCH


@app.route("/search", methods=["GET"])
@use_args(
    {
        "query": fields.Str(required=True),
    },
    location="query",
)
def search_all(args):
    return util.jsonify(search.all(util.get_user(required=True), args))


@app.route("/search/users", methods=["GET"])
@use_args(
    {
        "username": fields.Str(required=True),
    },
    location="query",
)
def search_users(args):
    return util.jsonify(search.users(util.get_user(required=True), args))


@app.route("/search/discover", methods=["GET"])
@use_args(
    {
        "count": fields.Int(),
    },
    location="query",
)
def search_discover(args):
    count = args.get("count", 3)
    if count:
        count = int(count)
    return util.jsonify(search.discover(util.get_user(required=False), count=count))


@app.route("/search/explore", methods=["GET"])
@use_args(
    {
        "page": fields.Int(),
    },
    location="query",
)
def search_explore(args):
    page = args.get("page", 1)
    if page:
        page = int(page)
    return util.jsonify(search.explore(page=page))


# INVITATIONS


@app.route("/invitations", methods=["GET"])
def invites_route():
    return util.jsonify(invitations.get(util.get_user(required=True)))


@app.route("/invitations/<id>", methods=["PUT", "DELETE"])
def invite_route(id):
    if request.method == "PUT":
        return util.jsonify(invitations.accept(util.get_user(required=True), id))
    if request.method == "DELETE":
        return util.jsonify(invitations.delete(util.get_user(required=True), id))


## ROOT


@app.route("/root/users", methods=["GET"])
def root_users():
    return util.jsonify(root.get_users(util.get_user(required=True)))


@app.route("/root/groups", methods=["GET"])
def root_groups():
    return util.jsonify(root.get_groups(util.get_user(required=True)))


## ActivityPub Support


@app.route("/.well-known/webfinger", methods=["GET"])
@use_args(
    {
        "resource": fields.Str(required=True),
    },
    location="query",
)
def webfinger(args):
    resource = args.get("resource")
    return util.jsonify(activitypub.webfinger(resource))


@app.route("/u/<username>", methods=["GET"])
def ap_user(username):
    resp_data = activitypub.user(username)
    resp = Response(json.dumps(resp_data))
    resp.headers["Content-Type"] = "application/activity+json"
    return resp


@app.route("/u/<username>/outbox", methods=["GET"])
@use_args(
    {
        "page": fields.Int(),
        "min_id": fields.Str(),
        "max_id": fields.Str(),
    },
    location="query",
)
def ap_user_outbox(args, username):
    page = args.get("page")
    min_id = args.get("min_id")
    max_id = args.get("max_id")
    resp_data = activitypub.outbox(username, page, min_id, max_id)
    resp = Response(json.dumps(resp_data))
    resp.headers["Content-Type"] = "application/activity+json"
    return resp
